@@ -3,6 +3,7 @@ Implements the RTS Target backstore and storage object classes.
 
 This file is part of RTSLib.
 Copyright (c) 2011-2013 by Datera, Inc
+Copyright (c) 2011-2014 by Red Hat, Inc.
 
 Licensed under the Apache License, Version 2.0 (the "License"); you may
 not use this file except in compliance with the License. You may obtain
@@ -107,15 +108,9 @@ class StorageObject(CFSNode):
         '''
         Build a StorageObject of the correct type from a configfs path.
         '''
-        mapping = dict(
-            fileio=FileIOStorageObject,
-            pscsi=PSCSIStorageObject,
-            iblock=BlockStorageObject,
-            rd_mcp=RDMCPStorageObject,
-            )
         so_name = os.path.basename(path)
         so_type = path.split("/")[-2].rsplit("_", 1)[0]
-        return mapping[so_type](so_name)
+        return so_mapping[so_type](so_name)
         
     def _get_wwn(self):
         self._check_self()
@@ -173,7 +168,7 @@ class StorageObject(CFSNode):
         self._check_self()
         info = fread("%s/info" % self.path)
         return re.search(".*%s: ([^: ]+).*" \
-                         % key, ' '.join(info.split())).group(1).lower()
+                         % key, ' '.join(info.split())).group(1)
 
     def _get_status(self):
         self._check_self()
@@ -671,14 +666,14 @@ class BlockStorageObject(StorageObject):
         if dev is not None:
             super(BlockStorageObject, self).__init__(name, 'create')
             try:
-                self._configure(dev, wwn, readonly, write_back)
+                self._configure(dev, wwn, readonly)
             except:
                 self.delete()
                 raise
         else:
             super(BlockStorageObject, self).__init__(name, 'lookup')
 
-    def _configure(self, dev, wwn, readonly, write_back):
+    def _configure(self, dev, wwn, readonly):
         self._check_self()
         if get_blockdev_type(dev) != 0:
             raise RTSLibError("Device is not a TYPE_DISK block device.")
@@ -690,9 +685,6 @@ class BlockStorageObject(StorageObject):
         self._control("readonly=%d" % readonly)
         self._enable()
 
-        if write_back:
-            self.set_attribute("emulate_write_cache", 1)
-
         super(BlockStorageObject, self)._configure(wwn)
 
     def _get_major(self):
@@ -760,6 +752,18 @@ class StorageObjectFactory(object):
         raise RTSLibError("Can't create storageobject from path: %s" % path)
 
 
+# Used to convert either dirprefix or plugin to the SO. Instead of two
+# almost-identical dicts we just have some duplicate entries.
+so_mapping = {
+    "pscsi": PSCSIStorageObject,
+    "rd_mcp": RDMCPStorageObject,
+    "ramdisk": RDMCPStorageObject,
+    "fileio": FileIOStorageObject,
+    "iblock": BlockStorageObject,
+    "block": BlockStorageObject,
+}
+
+
 bs_params = {
     PSCSIStorageObject: dict(name='pscsi'),
     RDMCPStorageObject: dict(name='ramdisk', alt_dirprefix='rd_mcp'),
@@ -816,7 +820,11 @@ class _Backstore(CFSNode):
         self._path = "%s/core/%s_%d" % (self.configfs_dir,
                                         dirp,
                                         self._index)
-        self._create_in_cfs_ine(mode)
+        try:
+            self._create_in_cfs_ine(mode)
+        except:
+            del bs_cache[self._lookup_key]
+            raise
 
     def delete(self):
         super(_Backstore, self).delete()
@@ -829,7 +837,7 @@ class _Backstore(CFSNode):
         self._check_self()
         info = fread("%s/hba_info" % self.path)
         return re.search(".*%s: ([^: ]+).*" \
-                         % key, ' '.join(info.split())).group(1).lower()
+                         % key, ' '.join(info.split())).group(1)
 
     def _get_version(self):
         self._check_self()
